//comp-func.g
//CHEMESIS1.0

function poolcyl (path, length, rad, initcon)
   str path
   float length
   float rad
   float initcon
   float   PI              =       3.14159
   float area = PI*rad*rad
   float volume = area * length / 1000	/* divide by 1000 to convert from cm^3 to L */
   float areaout = 2*PI*rad*length

   create rxnpool {path}
   setfield {path} \
	Cmin	0 \
	len	{length} \
	Conc	0.0e-3 \ 
	Cinit   {initcon} \	
	vol	{volume} \
	SAside	{area} \
	SAout   {areaout} \
	radius  {rad} \
	quantity 0 \
	Qinit	{initcon*volume} \
	type	0
end

/****************************************************************************/
function poolsphere (path, outerrad, innerrad, initcon)
   str path
   float outerrad
   float innerrad
   float initcon
   float   PI              =       3.14159
   float areaout = 4 * PI*outerrad*outerrad
   float areain = 4 * PI*innerrad*innerrad
   float volume = (areaout*outerrad - areain * innerrad) / 3 / 1000
		/* divide by 1000 to convert from cm^3 to L */

   create rxnpool {path}
   setfield {path} \
	Cmin	0 \
	Conc	0.0e-3 \ 
	Cinit   {initcon} \	
	vol	{volume} \
	SAout	{areaout} \
	SAin	{areain} \
	radius	{outerrad-innerrad} \
	quantity 0 \
	type	0 \
	Qinit {initcon*volume}
end

/****************************************************************************/
function pool2D (path, outerrad, innerrad, width, initcon)
   str path
   float outerrad
   float innerrad
   float initcon
   float width
   float   PI              =       3.14159
   float areaout = 2 * PI*outerrad*width
   float areain = 2 * PI*innerrad*width
   float areaside = PI * (outerrad*outerrad - innerrad * innerrad)
   float volume = areaside * width / 1000
	/* divide by 1000 to convert from cm^3 to L */

   create rxnpool {path}
   setfield {path} \
	Cmin	0 \
	Conc	0.0e-3 \ 
	Cinit   {initcon} \	
	vol	{volume} \
	SAout	{areaout} \
	SAin	{areain} \
	SAside  {areaside} \
	len	{width}	\
	radius	{outerrad-innerrad} \
	quantity 0 \
	Qinit	{initcon*volume} \
	type 0
end

/****************************************************************************/

function difsphere (path1,path2,difpath,dif)
   str path1, path2, difpath
   float dif
   
   create diffusion {difpath}
   setfield ^ \
	D {dif}

addmsg {path1} {difpath} POOL1 radius SAin Conc /* Should be "outer" pool */
addmsg {path2} {difpath} POOL2 radius SAout Conc /* Should be "inner" pool */
addmsg {difpath} {path1} DIFFUSION difconc2 difconc1
addmsg {difpath} {path2} DIFFUSION difconc1 difconc2

end
/********************************************************************/

function difcyl (path1,path2,difpath,dif)
   str path1, path2, difpath
   float dif
   
   create diffusion {difpath}
   setfield ^ \
	D {dif}

addmsg {path1} {difpath} POOL1 len SAside Conc /* Should be "left" pool */
addmsg {path2} {difpath} POOL2 len SAside Conc /* Should be "right" pool */
addmsg {difpath} {path1} DIFFUSION difconc2 difconc1
addmsg {difpath} {path2} DIFFUSION difconc1 difconc2

end

/****************************************************************************/

function compcyl (path, radius, ncyls, cylsize, calconc)
 float radius, cylsize
 float calconc
 int ncyls
 str path
 
 int cyl

 for (cyl = 1; cyl <= ncyls; cyl=cyl+1)
	poolcyl {path}[{cyl}] {cylsize} {radius} {calconc}
    end
end
/********************************************************************/

function compsphere(path, radius, nshells, shellsize, initcon)
  float radius, shellsize
  float initconc
  int nshells
  str path

  int shell
  float outer, inner

  outer=radius
  inner=radius-shellsize

  for (shell = 1; shell <= nshells; shell=shell+1)
	if (shell == nshells) ; inner = 0; end
	poolsphere {path}[{shell}] {outer} {inner} {initcon}
	outer = inner
	inner = inner - shellsize
  end
end
/********************************************************************/

function comp2D(path, radius, nshells, shellsize, len, ncyls, initcon)
  float radius, cylsize, len
  float initconc
  int nshells, ncyls
  str path

  int shell, cyl
  float outer, inner

  outer=radius
  inner=radius-shellsize

/* shell index first, cyl index second */

  for (shell = 1; shell <= nshells; shell=shell+1)
	if (shell == nshells) ; inner = 0; end

	for (cyl = 1; cyl <= ncyls; cyl = cyl+1)
	     pool2D {path}s{shell}[{cyl}] {outer} {inner} {len} {initcon}
	end

	outer = inner
	inner = inner - shellsize
  end
end

/********************************************************************/

function conservepool(path,Total, vol)
   str path
   float Total, vol

   create conservepool {path}
   setfield {path} \
	Ctot	{Total} \
	Cmin	0 \
	Conc	0 \
	Cinit	0 \
	Qinit	0 \
	quantity 0 \
	Qtot	{Total*vol} \
	type	0 \
	volume {vol}
end

/********************************************************************/

function consv1D (path, ncomp, total, radius, length)
  float total, radius, length
  str path
  int ncomp

  float volume = PI * radius*radius*length / 1000
	/* convert  from cm^3 to Liters */
  int i

  for (i = 1; i <= ncomp; i = i + 1)
    conservepool {path}[{i}] {total} {volume}
  end
end

/*********************************************************************/

function consv2D (path, nshell, ncyl, total, radius, length, shellsize)
  float total, radius, length, shellsize
  str path
  int ncyl, nshell

  float outer, inner, volume
  int i, j

  outer=radius
  inner=radius-shellsize

/* shell index first, cyl index second */

  for (i = 1; i <= nshell; i = i + 1)
   if (i == nshell) ; inner = 0; end

   float volume = PI * (outer*outer - inner * inner) * length / 1000
	/* divide by 1000 to convert from cm^3 to L */

    for (j = 1; j <= ncyl; j = j + 1)
       conservepool {path}s{i}[{j}] {total} {volume}
    end

    outer = inner
    inner = inner - shellsize
 
  end
end

/********************************************************************/
function difcomp2D(path, nshells, ncyls, difconst)
  float difconst
  int nshells, ncyls
  str path

  int shell, cyl

/* shell index first, cyl index second */
/*  axdif is diffusion axially, within a shell;
  raddif is difusion radially, within a cylinder */

  for (cyl = 1; cyl <= ncyls; cyl = cyl+1)

     for (shell = 1; shell < nshells; shell=shell+1)
        difsphere {path}s{shell}[{cyl}] {path}s{shell+1}[{cyl}] {path}_raddifs{shell}[{cyl}] {difconst}
     end
     if (cyl < ncyls)
       for (shell = 1; shell <= nshells; shell=shell+1)
          difcyl {path}s{shell}[{cyl}] {path}s{shell}[{cyl+1}] {path}_axdifs{shell}[{cyl}] {difconst}
       end
     end
  end
end

/********************************************************************/

function difcompcyl(path, ncyls, difconst)
  float difconst
  int ncyls
  str path

  int cyl

  for (cyl = 1; cyl < ncyls; cyl = cyl+1)
        difcyl {path}[{cyl}] {path}[{cyl+1}] {path}dif[{cyl}] {difconst}
  end
end

/********************************************************************/

function difcompsphere(path, nshells, difconst)
  float difconst
  int nshells
  str path

  int shell

  for (shell = 1; shell < nshells; shell = shell+1)
        difsphere {path}[{shell}] {path}[{shell+1}] {path}dif[{shell}] {difconst}
  end
end









































































































