//CHEMESIS1.0
//gaba-cytpump.g

/* specify all max conductances in microSiemens (uS),
        all voltages in millivolts (mV), resistances in Megaohms (Mohms) =>
        all currents are in nanoAmps (nA), lengths in centimeters (cm)
   specify capacitance in nF yields time in milliseconds */

/* Model of passive soma, rhabdomere, axon and terminal branches.
   Include GABA-A channel in terminal branches. */

include const.g
include rxn-func.g
include comp-func.g
include cicr-func.g
include ryan-func.g
include cicrflux-nER-func.g
include cal-ip3-nER.g
include volt-func.g
include gabaa20.g
include gabab-chan.g
include gabab-syn.g

int ERaxoncomps	=	10
maxcicr		=	0.4
kf_pip2 	= 	4.33e-18
axonwidth 	=	1e-4
branchwidth 	= 	1e-4
int branchcyls  =       15
int axoncyls    =       100     /*each slice is 5 cyls for calcium,etc.*/
int i, j

   /***** ****** set up soma *************************/
        /**voltage compartment **/

create conservepool /extracell
setfield /extracell Cinit 10 Ctot 10
useclock /extracell 2

create neutral /soma
Vcomp /soma/vm {somalen} {somarad} {RM} {CM} {RI}
ca_buf_ip3_ER /soma {somacyls} {somarad} {somalen} {ERfactor} 1
cytpumpcomp /soma/Cacyt /extracell pumphigh {somacyls} {Vmhigh} {KMhigh}
cytpumpcomp /soma/Cacyt /extracell pumplow {somacyls} {Vmlow} {KMlow}
 
  /************* set up rhabdomere **********************/
        /**voltage compartment **/

create neutral /rhab
Vcomp /rhab/vm {rhablen} {rhabrad} {RM} {CM} {RI}
setfield /rhab/vm Rm {RM/rhabSA} Cm {CM*rhabSA} Ra {RI*rhablen/rhabxarea}

  /************* create neck *************/
	/***voltage compartment **/
create neutral /neck
Vcomp /neck/vm {necklen} {neckrad} {RM} {CM} {RI}
 
   /************ set up axon *************************/
        /**voltage compartment **/

create neutral /axon
ellipse_vcomp /axon/vm {axonlen} {axonslice} {axonrada} {axonradb}  {RM} {CM} {RI}
ca_buf_ip3_ER /axon {axoncyls} {(axonrada+axonradb)/2} {axonwidth} {ERfactor} {ERaxoncomps}
cytpumpcomp /axon/Cacyt /extracell pumphigh {axoncyls} {Vmhigh} {KMhigh}
cytpumpcomp /axon/Cacyt /extracell pumplow {axoncyls} {Vmlow} {KMlow}

   /************ set up terminal branches *************************/
        /**voltage compartment **/
	/* assume 1 of num_branch branches has gaba synapse, others don't */

create neutral /branch_syn
Vcomp /branch_syn/vm[1] {branchlen1} {syn_br_rad} {RM} {CM} {RI}
Vcomp /branch_syn/vm[2] {branchlen2} {syn_br_rad} {RM} {CM} {RI}
ca_buf_ip3_ER /branch_syn {branchcyls} {syn_br_rad} {branchwidth} {ERfactor} 1
cytpumpcomp /branch_syn/Cacyt /extracell pumphigh {branchcyls} {Vmhigh} {KMhigh}
cytpumpcomp /branch_syn/Cacyt /extracell pumplow {branchcyls} {Vmlow} {KMlow}

create neutral /branch
Vcomp /branch/vm[1] {branchlen1} {nosyn_br_rad} {RM} {CM} {RI}
Vcomp /branch/vm[2] {branchlen2} {nosyn_br_rad} {RM} {CM} {RI}
ca_buf_ip3_ER /branch {branchcyls} {nosyn_br_rad} {branchwidth} {ERfactor} 1
cytpumpcomp /branch/Cacyt /extracell pumphigh {branchcyls} {Vmhigh} {KMhigh}
cytpumpcomp /branch/Cacyt /extracell pumplow {branchcyls} {Vmlow} {KMlow}

/* messages between voltage compartments */
addmsg /rhab/vm /neck/vm RAXIAL Ra previous_state
addmsg /neck/vm /rhab/vm AXIAL previous_state
addmsg /neck/vm /soma/vm RAXIAL Ra previous_state
addmsg /soma/vm /neck/vm AXIAL  previous_state
addmsg /soma/vm /axon/vm[1] RAXIAL Ra previous_state
addmsg /axon/vm[1] /soma/vm  AXIAL  previous_state
addmsg /axon/vm[{axonslice}] /branch_syn/vm[1] RAXIAL Ra previous_state
addmsg /axon/vm[{axonslice}] /branch/vm[1] RAXIAL Ra previous_state
addmsg /branch_syn/vm[1] /axon/vm[{axonslice}] AXIAL previous_state
addmsg /branch/vm[1] /axon/vm[{axonslice}] AXIAL previous_state
addmsg /branch_syn/vm[1] /branch_syn/vm[2] RAXIAL Ra previous_state
addmsg /branch_syn/vm[2] /branch_syn/vm[1] AXIAL previous_state
addmsg /branch/vm[1] /branch/vm[2] RAXIAL Ra previous_state
addmsg /branch/vm[2] /branch/vm[1] AXIAL previous_state

/* messages between rxnpool compartments */
difcyl /soma/ip3[1] /axon/ip3[1] /soma/axon_ip3dif {ip3dif}
difcyl /axon/ip3[{axoncyls}] /branch_syn/ip3[1] /axon/brsyn_ip3dif {ip3dif}
difcyl /axon/ip3[{axoncyls}] /branch/ip3[1] /axon/branch_ip3dif {ip3dif}
useclock /soma/axon_ip3dif 3
useclock /axon/brsyn_ip3dif 3
useclock /axon/branch_ip3dif 3

difcyl /soma/Cacyt[1] /axon/Cacyt[1] /soma/axon_Cacytdif {Cadif}
difcyl /axon/Cacyt[{axoncyls}] /branch_syn/Cacyt[1] /axon/brsyn_Cacytdif {Cadif}
difcyl /axon/Cacyt[{axoncyls}] /branch/Cacyt[1] /axon/branch_Cacytdif {Cadif}

/***************************************************/
/* set up statocyst as a spiker with adapting rate */

create neutral /stat
create randomspike /stat/spike
//setfield /stat/spike min_amp 1.5 max_amp 1.5 rate 0.025 abs_refract 20.0
setfield /stat/spike min_amp 1.0 max_amp 1.0 rate 0.2 abs_refract 4.0
create transmitter /stat/spike_rate
setfield /stat/spike_rate peak 0.15 tau_fall 1000 decay_type 0 delay 1000
addmsg /stat/spike_rate /stat/spike RATE trans_conc


/************************GABA synapse*******************/
str gaba="/branch_syn/vm[2]"

/**** make gabaa channel in the synaptic branch ****/
makegabaa {gaba} /stat/spike {g_gabaa}

/* make gabab receptor, biochem reactions */
gabab_synapse {gaba} {branchlen2/2} {syn_br_rad}

/* make gabab channel */
makegabab {gaba} /branch_syn/vm[2]/Galfstar {g_gabab}

float vol = PI*syn_br_rad*syn_br_rad*branchlen2/2/1000

/************ Activation of IICR via PLC and IP3 *****************/
/* plc + galfstar <-> plc*-g* */
/* Neither Galfstar bound to plc nor Galfstar bound to gabab channel is taken 
into account in computing Gprot conservepool concentration */
create conservepool {gaba}/plctot	/*pool of plc, inactive */
setfield {gaba}/plctot Cinit {plc_tot} Ctot {plc_tot} volume {vol} Qtot {plc_tot*vol} Qinit {plc_tot*vol} type 0
create rxnpool {gaba}/plcGa /*pool of active plc */
setfield {gaba}/plcGa Cinit 0 Qinit 0 vol {vol} type 0 /*pool of active plc */
create reaction {gaba}/plc_rxn
setfield {gaba}/plc_rxn kf {plc_kf} kb {plc_kb}
addmsg {gaba}/Galfstar {gaba}/plc_rxn SUBSTRATE Conc
addmsg {gaba}/plctot {gaba}/plc_rxn SUBSTRATE Conc
addmsg {gaba}/plcGa {gaba}/plc_rxn PRODUCT Conc
addmsg {gaba}/plc_rxn {gaba}/plcGa RXN2 kfsubs kbprod
addmsg {gaba}/plcGa {gaba}/plctot CONC Conc

int syn_comp = {branchlen2/branchwidth}
/* plc + pip2 <-> ip3 + plc using mm kinetics */
   create conservepool {gaba}/pip2	/* pool of PIP2 substrate */
   setfield {gaba}/pip2 Cinit {pip2tot} Ctot {pip2tot} volume {vol} Qtot {pip2tot*vol} Qinit {pip2tot*vol}
   create enzyme {gaba}/pip2ip3 
/* Assign divide enzymatic activity by the number of IP3 production compartments*/
   setfield {gaba}/pip2ip3 Vmax {kf_pip2/syn_comp} Km {km_pip2}
   addmsg {gaba}/plcGa {gaba}/pip2ip3 ENZ Conc
   addmsg {gaba}/pip2 {gaba}/pip2ip3 SUBSTRATE Conc 

int start = {branchcyls} - {syn_comp} + 1
for (i=start; i<=branchcyls; i=i+1)
   addmsg {gaba}/pip2ip3 /branch_syn/ip3[{i}] MMPUMP product
   addmsg /branch_syn/ip3[{i}] {gaba}/pip2 CONC Conc
end

/* Add pulse current injection to measure input resistance before and 
	during IPSP*/
/*create pulsegen /soma/inject
setfield /soma/inject width1 200 level1 -0.6 baselevel 0 delay1 500 trig_mode 0
addmsg /soma/inject /soma/vm INJECT output
*/

/*include graphs.g*/

create asc_file /output/plot_out
addmsg /branch_syn/Cacyt[1] /output/plot_out SAVE Conc	/*2*/
addmsg /branch_syn/Cacyt[5] /output/plot_out SAVE Conc	/*3*/
addmsg /branch/Cacyt[1] /output/plot_out SAVE Conc	/*4*/
addmsg /branch/Cacyt[5] /output/plot_out SAVE Conc	/*5*/
addmsg /axon/Cacyt[100] /output/plot_out SAVE Conc	/*6*/
addmsg /axon/Cacyt[98] /output/plot_out SAVE Conc	/*7*/
addmsg /axon/Cacyt[96] /output/plot_out SAVE Conc	/*8*/
addmsg /axon/Cacyt[94] /output/plot_out SAVE Conc	/*9*/
addmsg /axon/Cacyt[92] /output/plot_out SAVE Conc	/*10*/
addmsg /axon/Cacyt[90] /output/plot_out SAVE Conc	/*11*/
addmsg /axon/Cacyt[88] /output/plot_out SAVE Conc	/*12*/
addmsg /axon/Cacyt[86] /output/plot_out SAVE Conc	/*13*/
addmsg /axon/Cacyt[84] /output/plot_out SAVE Conc	/*14*/
addmsg /axon/Cacyt[82] /output/plot_out SAVE Conc	/*15*/
addmsg /axon/Cacyt[80] /output/plot_out SAVE Conc	/*16*/
addmsg /axon/Cacyt[78] /output/plot_out SAVE Conc	/*17*/
addmsg /axon/Cacyt[76] /output/plot_out SAVE Conc	/*18*/
addmsg /axon/Cacyt[74] /output/plot_out SAVE Conc	/*19*/
addmsg /axon/Cacyt[72] /output/plot_out SAVE Conc	/*20*/
addmsg /axon/Cacyt[70] /output/plot_out SAVE Conc	/*21*/
addmsg /axon/Cacyt[68] /output/plot_out SAVE Conc	/*22*/
addmsg /axon/Cacyt[66] /output/plot_out SAVE Conc	/*23*/
addmsg /axon/Cacyt[64] /output/plot_out SAVE Conc	/*24*/
addmsg /axon/Cacyt[62] /output/plot_out SAVE Conc	/*25*/
addmsg /axon/Cacyt[60] /output/plot_out SAVE Conc	/*26*/
addmsg /axon/Cacyt[58] /output/plot_out SAVE Conc	/*27*/
addmsg /axon/Cacyt[56] /output/plot_out SAVE Conc	/*28*/
addmsg /axon/Cacyt[54] /output/plot_out SAVE Conc	/*29*/
addmsg /axon/Cacyt[52] /output/plot_out SAVE Conc	/*30*/
addmsg /axon/Cacyt[50] /output/plot_out SAVE Conc	/*31*/
addmsg /axon/Cacyt[48] /output/plot_out SAVE Conc	/*32*/
addmsg /axon/Cacyt[46] /output/plot_out SAVE Conc	/*33*/
addmsg /axon/Cacyt[44] /output/plot_out SAVE Conc	/*34*/
addmsg /axon/Cacyt[42] /output/plot_out SAVE Conc	/*35*/
addmsg /axon/Cacyt[40] /output/plot_out SAVE Conc	/*36*/
addmsg /axon/Cacyt[38] /output/plot_out SAVE Conc	/*37*/
addmsg /axon/Cacyt[36] /output/plot_out SAVE Conc	/*38*/
addmsg /axon/Cacyt[34] /output/plot_out SAVE Conc	/*39*/
addmsg /axon/Cacyt[32] /output/plot_out SAVE Conc	/*40*/
addmsg /axon/Cacyt[30] /output/plot_out SAVE Conc	/*41*/
addmsg /axon/Cacyt[28] /output/plot_out SAVE Conc	/*42*/
addmsg /axon/Cacyt[26] /output/plot_out SAVE Conc	/*43*/
addmsg /axon/Cacyt[24] /output/plot_out SAVE Conc	/*44*/
addmsg /axon/Cacyt[22] /output/plot_out SAVE Conc	/*45*/
addmsg /axon/Cacyt[20] /output/plot_out SAVE Conc	/*46*/
addmsg /axon/Cacyt[18] /output/plot_out SAVE Conc	/*47*/
addmsg /axon/Cacyt[16] /output/plot_out SAVE Conc	/*48*/
addmsg /axon/Cacyt[14] /output/plot_out SAVE Conc	/*49*/
addmsg /axon/Cacyt[12] /output/plot_out SAVE Conc	/*50*/
addmsg /axon/Cacyt[10] /output/plot_out SAVE Conc	/*51*/
addmsg /axon/Cacyt[8] /output/plot_out SAVE Conc	/*52*/
addmsg /axon/Cacyt[6] /output/plot_out SAVE Conc	/*53*/
addmsg /axon/Cacyt[4] /output/plot_out SAVE Conc	/*54*/
addmsg /axon/Cacyt[2] /output/plot_out SAVE Conc	/*55*/
addmsg /soma/Cacyt[1] /output/plot_out SAVE Conc	/*56*/
addmsg /soma/CaER[1] /output/plot_out SAVE Conc		/*57*/
addmsg /axon/CaER[1] /output/plot_out SAVE Conc		/*58*/
addmsg /axon/CaER[4] /output/plot_out SAVE Conc		/*59*/
addmsg /axon/CaER[7] /output/plot_out SAVE Conc		/*60*/
addmsg /axon/CaER[10] /output/plot_out SAVE Conc	/*61*/
addmsg /axon/Cacyt[70]/x110 /output/plot_out SAVE fraction	/*62*/
addmsg /axon/Cacyt[70]/x10 /output/plot_out SAVE fraction	/*63*/
addmsg /axon/Cacyt[40]/x110 /output/plot_out SAVE fraction	/*64*/
addmsg /axon/Cacyt[40]/x10 /output/plot_out SAVE fraction	/*65*/
addmsg /axon/Cacyt[20]/x110 /output/plot_out SAVE fraction	/*66*/
addmsg /axon/Cacyt[20]/x10 /output/plot_out SAVE fraction	/*67*/
addmsg /soma/Cacyt[1]/x110 /output/plot_out SAVE fraction	/*68*/
addmsg /soma/Cacyt[1]/x10 /output/plot_out SAVE fraction 	/*69*/

useclock /output/plot_out 2

setclock 1 0.001	/* Used for Calcium */
setclock 0 0.01
setclock 2 100.0
setclock 3 0.1		/* Used for ip3*/
setclock 4 0.0002	/* Used for soma Calcium */
useclock /soma/CaER[1] 4
useclock /soma/bufER[1] 4
useclock /soma/bufbndER[1] 4
setmethod /soma/CaER[1] -1
setmethod /soma/bufER[1] -1
setmethod /soma/bufbndER[1] -1

useclock /stat/spike 2
useclock /soma/inject 2

float pump, sercarate
float rate = 0.15
float plc = kf_pip2
float area, iicr_D, cicr_D

  for (maxcicr=0.4; maxcicr <= 0.6; maxcicr = maxcicr+0.2)
    for (i=1; i<=axoncyls; i=i+1)
	area={getfield /axon/Cacyt[{i}] vol}
	iicr_D=maxiicr*area
	cicr_D=maxcicr*area
	setfield /axon/Cacyt[{i}]/iicrflux maxflux {iicr_D}
	setfield /axon/Cacyt[{i}]/ryanflux maxflux {cicr_D}
    end
    for (i=1; i<=branchcyls; i=i+1)
	area={getfield /branch/Cacyt[{i}] vol}
	iicr_D=maxiicr*area
	cicr_D=maxcicr*area
	setfield /branch/Cacyt[{i}]/iicrflux maxflux {iicr_D}
	setfield /branch/Cacyt[{i}]/ryanflux maxflux {cicr_D}
	area={getfield /branch_syn/Cacyt[{i}] vol}
	iicr_D=maxiicr*area
	cicr_D=maxcicr*area
	setfield /branch_syn/Cacyt[{i}]/iicrflux maxflux {iicr_D}
	setfield /branch_syn/Cacyt[{i}]/ryanflux maxflux {cicr_D}
    end
    for (i=1; i<=somacyls; i=i+1)
	area={getfield /soma/Cacyt[{i}] vol}
	iicr_D=maxiicr*area
	cicr_D=maxcicr*area
	setfield /soma/Cacyt[{i}]/iicrflux maxflux {iicr_D}
	setfield /soma/Cacyt[{i}]/ryanflux maxflux {cicr_D}
    end

for (sercarate=0.004; sercarate >= 0.002; sercarate = sercarate-0.001)
   setsercaleak /branch/Cacyt /branch/CaER 0 {branchcyls} {sercarate} 1
   setsercaleak /branch_syn/Cacyt /branch_syn/CaER 0 {branchcyls} {sercarate} 1
   setsercaleak /axon/Cacyt /axon/CaER 0 {axoncyls} {sercarate} {ERaxoncomps}
   setsercaleak /soma/Cacyt /soma/CaER 0 {somacyls} {sercarate} 1

   for (pump=0.001e-6; pump>0.001e-8; pump=pump/5)
     setcytpumpleak /branch/Cacyt /extracell {branchcyls} {pump} {KMhigh} {Vmlow} {KMlow}
     setcytpumpleak /branch_syn/Cacyt /extracell {branchcyls} {pump} {KMhigh} {Vmlow} {KMlow}
     setcytpumpleak /axon/Cacyt /extracell {axoncyls} {pump} {KMhigh} {Vmlow} {KMlow}
     setcytpumpleak /soma/Cacyt /extracell {somacyls} {pump} {KMhigh} {Vmlow} {KMlow}

	echo axon serca: {getfield /axon/Cacyt[{axoncyls}]/serca max_rate} cytleak: {getfield /axon/Cacyt[{axoncyls}]/cytleak maxflux} cytpump: {getfield /axon/Cacyt[{axoncyls}]/pumphigh max_rate}  
	echo nosyn: serca: {getfield /branch/Cacyt[{branchcyls}]/serca max_rate} leak: {getfield /branch/Cacyt[{branchcyls}]/cytleak maxflux} cytpump: {getfield /branch/Cacyt[{branchcyls}]/pumphigh max_rate}   
	echo syn: serca: {getfield /branch_syn/Cacyt[{branchcyls}]/serca max_rate} cytleak: {getfield /branch_syn/Cacyt[{branchcyls}]/cytleak maxflux} cytpump: {getfield /branch_syn/Cacyt[{branchcyls}]/pumphigh max_rate}  
 	echo soma: serca: {getfield /soma/Cacyt[{somacyls}]/serca max_rate} cytleak: {getfield /soma/Cacyt[{somacyls}]/cytleak maxflux} cytpump: {getfield /soma/Cacyt[{somacyls}]/pumphigh max_rate}  

         str filenam = "/home/avrama/genesis/gaba1.0/cytpump/cicr"@(maxcicr)@"-serca"@(sercarate)@"-pump"@(pump)@".dat"
         setfield /output/plot_out filename {filenam} initialize 1 append 1 leave_open 1

        setfield /stat/spike_rate peak {rate} delay 200
        reset
        setclock 0 0.001
	setclock 4 0.00005
        step 1000
	setclock 0 0.01
	setclock 4 0.0002
        step 9900
        step 300000
        setfield /stat/spike_rate peak 0
        step 500000 
    end
  end
end




