//CHEMESIS1.0
//rxn-func.g
// functions to create pools and setup reactions


function make2ndorderrxn(sub1,sub2,prod,rxn,kfor,kbac,type)

/* This is used for calcium (sub1) binding to buffer (sub2); 
** product (bound buffer) is conserved if type=1*/

   str sub1,sub2,prod,rxn
   float kfor,kbac
   int type
   
   create reaction {rxn} 
   setfield ^ \
	kf	{kfor}	\/* per mM-mSec.*/
	kb	{kbac}	 /* per mSec */

/* Send substrate and product conc to reaction */
addmsg {sub1} {rxn} SUBSTRATE Conc
addmsg {sub2} {rxn} SUBSTRATE Conc
addmsg {prod} {rxn} PRODUCT Conc

/* Send A and B state variables to substrates */
addmsg {rxn} {sub1} RXN2 kbprod kfsubs
addmsg {rxn} {sub2} RXN2 kbprod kfsubs
if (type == 1)
  addmsg {sub2} {prod} CONC Conc
end
if (type == 0)
  addmsg {rxn} {prod} RXN2 kfsubs kbprod
end
end
/********************************************************************/

function rxncomp2D (sub1,sub2,prod,rxn,nshell, ncyl,kfor,kbac,type)

str sub1, sub2, prod, rxn
float kfor,kbac
int ncyl,nshell
int type

int cyl, shell

/* shell index first, cyl index second */

  for (cyl=1; cyl<=ncyl; cyl=cyl+1)
    for (shell=1; shell<=nshell; shell=shell+1)
      make2ndorderrxn {sub1}s{shell}[{cyl}] {sub2}s{shell}[{cyl}] {prod}s{shell}[{cyl}] {rxn}s{shell}[{cyl}] {kfor} {kbac} {type}
    end
  end
end
/********************************************************************/

function rxncomp1D (sub1,sub2,prod,rxn,ncyl,kfor,kbac,type)

str sub1, sub2, prod, rxn
float kfor,kbac
int ncyl
int type

int cyl

  for (cyl=1; cyl<=ncyl; cyl=cyl+1)
      make2ndorderrxn {sub1}[{cyl}] {sub2}[{cyl}] {prod}[{cyl}] {rxn}[{cyl}] {kfor} {kbac} {type}
  end

end
/********************************************************************/


function makepump(cytpath,erpath,maxrate,expon,halfconc)
   str cytpath, erpath
   float maxrate, halfconc
   int expon
   
   create mmpump {cytpath}/serca
   setfield ^ \
      power {expon} \
      half_conc {halfconc} \
      max_rate {maxrate}
      
addmsg {cytpath} {cytpath}/serca CONC Conc
addmsg {cytpath}/serca {cytpath} MMPUMP moles_out
addmsg {cytpath}/serca {erpath} MMPUMP moles_in

end

/********************************************************************/

function cytpumpcomp (cytcomp, extracell,pumptype, ncyl, vmax, km)
float vmax, km
str cytcomp, pumptype, extracell
int ncyl

float area, pumprate
int cyl

  for (cyl=1; cyl<=ncyl; cyl=cyl+1)

/* compute value of cytosolic pump from area */

	area = {getfield {cytcomp}[{cyl}] SAout}
	pumprate = vmax *  area

/* create cytosolic mmpump */

	create mmpump {cytcomp}[{cyl}]/{pumptype}
	setfield ^ \
		power 2 \
		half_conc {km} \
		max_rate {pumprate}

	addmsg {cytcomp}[{cyl}] {cytcomp}[{cyl}]/{pumptype} CONC Conc
	addmsg {cytcomp}[{cyl}]/{pumptype} {cytcomp}[{cyl}] MMPUMP moles_out
 
/* create leak from extracellular space to cytoplasm, if not already created */

     if (!{exists {cytcomp}[{cyl}]/cytleak})
	create cicrflux {cytcomp}[{cyl}]/cytleak
	setfield ^ power 1 maxflux 1

	addmsg {cytcomp}[{cyl}] {cytcomp}[{cyl}]/cytleak CONC1 Conc
	addmsg {extracell} {cytcomp}[{cyl}]/cytleak CONC2 Conc
	addmsg {cytcomp}[{cyl}] {cytcomp}[{cyl}]/cytleak IP3R 1
	addmsg {cytcomp}[{cyl}]/cytleak {cytcomp}[{cyl}] DIFFUSION fluxconc2 fluxconc1
     end
  end

end
/********************************************************************/
function makeleak(cytpath,erpath,maxrate,expon)
   str cytpath, erpath
   float maxrate
   int expon
   
   create cicrflux {cytpath}/leak
   setfield ^ \
      power {expon} \
      maxflux {maxrate}
      
addmsg {cytpath} {cytpath}/leak CONC1 Conc
addmsg {erpath} {cytpath}/leak CONC2 Conc
addmsg {cytpath} {cytpath}/leak IP3R 1
addmsg {cytpath}/leak {cytpath} DIFFUSION fluxconc2 fluxconc1
addmsg {cytpath}/leak {erpath} DIFFUSION fluxconc1 fluxconc2

end
/********************************************************************/
/* this is a new makenzyme function.  The PLC value during the light is 
  the sum of a pedestal value and a decaying value. */

function makenzyme(enzpath, plcvalue, plctime, plcbase, plc_on, enzrxnpath, kfvalue, prodpath, enztranspath, plcconst, plcdecay, type)
   str enzrxnpath, prodpath, enzpath, enztranspath
   float kfvalue, degrad, plcvalue, plctime, plcdecay
   int type

   create pulsegen {enzpath}
   setfield {enzpath}      \
	  baselevel {plcbase}  \ 
          level1 {plcvalue*plcconst} \ /* PLC value during light on */
          width1 {plctime} \ /* specify in units of time, not steps */
          delay1 {plc_on} \ /* specify in units of time, not steps */
          delay2 99999  \
          trig_mode 0 \
          trig_time 0 

  create transmitter {enztranspath}
  setfield  {enztranspath} \
	decay_type {type} \ /* decay type 0 is exp, type 1 is linear */
	delay {plc_on} \
	tau_fall {plcdecay} \
	peak {plcvalue*(1-plcconst)}

/*	  baselevel  0.125e-10: previous PLC value during light off -from Suzuki et al. 1995 - assume plc = 1/100th quant of G prot*/

   create enzyme {enzrxnpath}
   setfield {enzrxnpath} Km {kfvalue}
   
   addmsg {enzpath} {enzrxnpath} ENZ output
   addmsg {enztranspath} {enzrxnpath} ENZ trans_conc
 
   addmsg {enzrxnpath} {prodpath} MMPUMP product

end

/**************************************************************/

function makedegrad(path, pool, numcyls, degrad)
str path
int numcyls
float degrad

int cyl

   create reaction {path}{pool}degrad
   setfield ^ kf {degrad}

   for (cyl=1; cyl<=numcyls; cyl=cyl+1)
       addmsg {path}{pool}degrad {path}{pool}[{cyl}] RXN1 kf
   end
end

/*********************************************************************/

function degrad2D (path, pool, nshell, ncyl, degrad)

str path, pool
int ncyl, nshell
float degrad

int cyl, shell

   create reaction {path}{pool}degrad
   setfield ^ kf {degrad}

/* shell index first, cyl index second */

  for (cyl=1; cyl<= ncyl; cyl=cyl+1)
    for (shell=1; shell<=nshell; shell=shell+1)
	addmsg {path}{pool}degrad {path}{pool}s{shell}[{cyl}] RXN1 kf
    end
  end

end
















