// genesis
// chanlib.g
/************************************************************************
**                      SHELL COMMANDS                    	     **
************************************************************************
*/

/*
*************************************************************************
**                              OBJECT TABLE                           **
*************************************************************************
*/

//==========================================================================
//	name		data            function	class	
//==========================================================================
// under list of messages, the value 0, 1, etc., immediately following the all 
// caps NAME of the message must correspond with the case number in MSGLOOP 
// of the code.  The short name, e.g. Vm, at the end of the message line
// must correspond to ?. The caps NAME of the message must correspond to
// the caps NAME in the sendmsg statement. (the variable name following the 
// caps NAME in the sendmsg statement must be a defined field
// in the object structure.)
//
//     object name	type (*.h)		function (*.c)      class

object transmitter 	transmitter_type 	transmitter segment channel\
	-author		"Avrama Blackwell ERIM 04/95 "\
	-actions	RESET PROCESS \
	-fields		"trans_conc value of transmitter at time t" \
			"peak peak value of transmitter" \
			"tau_fall: decay time of transmitter" \
  			"decay_type: 0 for exponential, 1 for linear " \
  			"delay: time of arrival of transmitter" \
	-description	" Computes transmitter concentration when the pre-synaptic element" \
			" is not coded explicitly.  Transmitter is assumed to arrive all at once." \
			" decay is either exponential (0) or linear (1)."


object ligand1_chan 	ligand1_chan_type 	ligand1_chan segment channel\
	-author		"Avrama Blackwell ERIM 05/95 "\
	-actions	INIT RESET PROCESS CHECK SAVE2 RESTORE2 \
	-messages       VOLTAGE 0 1 Vm \
                        LIGAND 1 1 ligand \
	-fields		"alpha: structure - rate const of binding " \
			"beta: structure - rate const of unbinding " \
			"gamma: structure - rate const of binding for inact" \
			"delta: structure - rate const of unbinding for inact" \
			"inact_ss: structure - for v dep inactivation gate " \
			"inact_tau: structure - for v dep inactivation gate " \
			"inact_type: 0 for v dep inf and tau, 1 for v dep, lig dep gamma and delta" \
			"ligand: transmitter (or other 2nd messenger) concentration in mM " \
			"act_power: exponent indicated # independent gates " \ 
			"inact_power: exponent indicating # independent gates " \
			"rxn_ord:  number of molecules binding to receptor " \
			"inact_rxn_ord:  number of molecules binding to inactivation molecule " \
			"Vr: reversal potential of channel" \
			"Gbar: Maximum channel conductance" \
			"Vm: voltage sensed by channel" \
			"I:  channel current " \
			"G:  Channel conductance" \
			"act_inf: ss activation computed from alpha and beta" \
			"act_tau: time const of act computed from alpha and beta" \
			"inact_inf:  ss inact computed from gamma and delta" \
			"inact_rate: ss rate of inact from gamma and delta" \
			"act: computed activation" \
			"inact: computed inactivation" \
	-description	" This function computes channel conductance of a one state kinetic " \
			" channel which is ligand-gated (and possibly voltage-dependent) according to: " \
			"   [R] + n[G] <-> [R-G^n] " \
			"   where n is the number of molecules of ligand G that binds to the " \
			"   receptor.  alpha and beta are forward and backward rate constant. " \
			"  inact_ss and inact_tau are rate constants for optional inact gate. "\
			" gamma and delta are forward and backward rate constants for " \
			" optional ligand dependent inact gate. (ligand multiplies delta" \
			"all rate constant structures have the following fields: rate_const(v) = " \
			"min + max * {out_exp_offset + exp[((v - v0)/slope)^in_exp_power + in_exp_offset]} " \
			" ^out_exp_power " \
 			" *	for hh channels (Sigmoid type), " \
			" *		out_exp_power = -1,  " \
			" *		in_exp_power = 1, " \
			" *		out_exp_offset = 1, " \
			" *		in_exp_offset = 0 " \
			" *      for voltage-independent channels make " \
			" *               max = 0, " \
			" *               min = desired rate constant " \
			" *               slope != 0 "  \
			" Units are milliVolts, nanoAmps, microSiemens, milliSeconds, millimoles"


object ligand2_chan 	ligand2_chan_type 	ligand2_chan segment channel\
        -author         "Avrama Blackwell ERIM 04/95 "\
        -actions        INIT RESET PROCESS CHECK \
        -messages       VOLTAGE 0 1 Vm \
                        LIGAND 1 1 ligand \
	-fields		"k1 structure - rate const of binding " \
			"k2 structure - rate const of unbinding " \
			"alpha structure - vdep transformation of 2nd reaction " \
			"beta structure - vdep transformation of 2nd reaction" \
			"gamma structure - vdep transformation of 3d reaction " \
			"delta structure - vdep transformation of 3d reaction" \
			"ligand = transmitter (or other 2nd messenger) concentration in mM " \
			"rxn_ord = number of molecules binding to receptor " \
			"Vr: reversal potential of channel" \
			"Gbar: Maximum channel conductance" \
			"Vm: voltage sensed by channel" \
			"I:  channel current"  \
			"G:  Channel conductance" \
	-description	" This function computes channel conductance of a two state kinetic channel " \
			" which is voltage and ligand dependent.  It computes the fraction of " \
			" receptors in each state; the channel conductance and current is computed " \
			" from the fraction of receptors in the conducting state. " \
			" Equations implemented in this function: " \
			"                         	 k1      alpha " \
			"   Transmitter/Ligand + Receptor <-> T-R1 <-> T-R2 " \
			"                          	k2      beta " \
			"            gamma" \
			" Also, T-R2 <-> T + R " \
			"            delta " \
			"all rate constant structures have the following fields: rate_const(v) = " \
			"min + max * {out_exp_offset + exp[((v - v0)/slope)^in_exp_power + in_exp_offset]} " \
			" ^out_exp_power " \
 			" *	for hh channels (Sigmoid type), " \
			" *		out_exp_power = -1,  " \
			" *		in_exp_power = 1, " \
			" *		out_exp_offset = 1, " \
			" *		in_exp_offset = 0 " \
			" *      for voltage-independent channels make " \
			" *               max = 0, " \
			" *               min = desired rate constant " \
			" *               slope != 0 "  \
			" Units are milliVolts, nanoAmps, microSiemens, milliSeconds, millimoles"

                          
object inf_tau_chan 	vdep_chan_type 	inf_tau_chan segment channel\
	-author		"Avrama Blackwell ERIM 05/95 "\
	-actions	INIT RESET PROCESS CHECK SAVE2 RESTORE2 \
	-messages       VOLTAGE 0 1 Vm \
	-fields		"act_ss: structure - for v dep activation gate " \
			"act_tau: structure - for v dep activation gate " \
			"inact_ss: structure - for v dep inactivation gate " \
			"inact_tau: structure - for v dep inactivation gate " \
			"act_power: exponent indicated # independent gates " \ 
			"inact_power: exponent indicating # independent gates " \
			"Vr: reversal potential of channel (milliVolts)" \
			"Gbar: Maximum channel conductance (microSiemens)" \
			"Vm: voltage sensed by channel (milliVolts)" \
			"I:  channel current (nanoAmps)" \
			"G:  Channel conductance (microSiemens)" \
			"act:  computed activation" \
			"inact: computed inactivation " \
	-description	" This  program computes voltage dependent channel currents given " \
			" the parameters specified as inf (steady state value) and tau (time const)."\
			"all rate constant structures have the following fields: rate_const(v) = " \
			"min + max * {out_exp_offset + exp[((v - v0)/slope)^in_exp_power + in_exp_offset]} " \
			" ^out_exp_power " \
 			" *	for hh channels (Sigmoid type), " \
			" *		out_exp_power = -1,  " \
			" *		in_exp_power = 1, " \
			" *		out_exp_offset = 1, " \
			" *		in_exp_offset = 0 " \
			" *      for voltage-independent channels make " \
			" *               max = 0, " \
			" *               min = desired rate constant " \
			" *               slope != 0 " \
			" Units are milliVolts, nanoAmps, microSiemens, milliSeconds"

object vdep_ligdep_chan 	vdep_ligdep_chan_type 	vdep_ligdep_chan segment channel\
	-author		"Avrama Blackwell ERIM 05/95 "\
	-actions	INIT RESET PROCESS CHECK SAVE2 RESTORE2 \
	-messages       VOLTAGE 0 1 Vm \
			LIGAND 1 1 ligand \
	-fields		"act_ssv: structure - for v dep activation gate " \
			"act_tauv: structure - for v dep activation gate " \
			"inact_ssv: structure - for v dep inactivation gate " \
			"inact_tauv: structure - for v dep inactivation gate " \
			"act_ssca: structure - for ligand dep activation gate " \
			"act_tauca: structure - for ligand dep activation gate " \
			"inact_ssca: structure - for ligand dep inactivation gate " \
			"inact_tauca: structure - for ligand dep inactivation gate " \
			"act_power: exponent indicated # independent gates " \ 
			"inact_power: exponent indicating # independent gates " \
			"type: type of interaction of ligand and Voltage" \
			"Vr: reversal potential of channel" \
			"Gbar: Maximum channel conductance" \
			"Vm: voltage sensed by channel" \
			"I:  channel current " \
			"G:  Channel conductance" \
	-description	" This function computes voltage-dependent and ligand-dependent (e.g. Calcium " \
			"dependent) channel currents given the parameters specified as inf (steady state value) and tau (time const)." \
			" Type of rate constants depends on the interaction of ligand and voltage: " \
		    	" 0 = separable (vdep coef * ligdep coef) " \
		    	" 1 = nonseparable linear: (vdep coef = int + slp * ligand " \
		    	" 2 = nonseparable log: (vdep coef = int + slp * log(ligand) " \
		    	" 3 = nonseparable sigmoid: (vdep coef =  slp * ligand / (int + ligand)" \
			"all rate constant structures have the following fields: rate_const(v) = " \
			"min + max * offset + exp[(v - v0)/slope]}^power,   where: " \
			"  min = min_slp * f(Ca) + min_int " \
			"  max = max_slp * f(Ca) + max_int " \
			"  v0 = v0_slp * f(Ca) + v0_int " \
			"  slope = slope_slp * f(Ca) + slope_int " \
 			" *   for hh channels (Sigmoid type), " \
			" *		power = -1,  " \
			" *		offset = 1, " \
			" *   for separable,  " \
			"	use v dep and ligand dep gates;	make slp = 0 and int = desired value " \
			" *   for non-separable, use v dep gate only; specify slp and int"  \
			" Units are milliVolts, nanoAmps, microSiemens, milliSeconds, millimoles"


object  electrodif	 GHK_type 	electrodif	segment channel\
        -author         "Avrama Blackwell GMU 03/98 "\
        -actions        INIT RESET PROCESS \
        -messages       CONC_EXT 0 1 ConcExt \
                        CONC_INT 1 1 ConcInt \
                        VOLTAGE  2 1 Vm \
                        OPEN_FRACTION 3 2 G Gbar \
        -fields         "conc_ext: external concentration of substance " \
                        "conc_int: internal concentration of substance " \
                        "Vm: voltage sensed by channel (in millivolts)" \
                        "Vr: reversal potential calculated by Nernst Eqn. " \
                        "pseudoG: conductance to give equivalent current with Ohm's law " \
                        " I: current through channel " \
                        " Pca: permeability of channel " \
                        " charge: charge of ion permeating channel  " \
                        " chi: RT/zF - stored to minimize computations " \
                        " T: temperature in kelvin " \
        -description    "computes current using the Goldman-Hodgkin-Katz Equation" \
                        "Then, computes an equivalent G and Vr to send to the compartment " \
                        "object.  Allows for modification of the GHK current by open fraction. " \
                        "Implements following equation:" \
                        "define chi = (z*F)/(R*T*1000),    define coef = Pca*z*F*chi " \
                        "I(V) = coef*open*v*(Conc_int - conc_ext*exp(-chi*v))/(1-exp(-chi*v))"
 

