/* chem_struct.h */

#include "struct_defs.h"

/*
*************************************
**         CHEMICAL REACTIONS      **
** Avrama Blackwell, GMU, 1997     **
** modified December, 1999         **
*************************************
*/

struct conserve_type {
    SEGMENT_TYPE
    double previous_state;
    double Conc;
    double quantity;
    double Qinit;
    double Qtot;
    double Cinit;
    double Cmin;
    double Ctot;
    double volume;
    int type;
};

struct react_type {
    SEGMENT_TYPE
    double kb;
    double kf;
    double kbprod, kfsubs;
};

struct enzyme_type {
    SEGMENT_TYPE
    double Vmax;
    double Km;
    double product;
    float thresh;
    int pos_pow;
    int pos_form;
    float pos_halfmax;
    int neg_pow;
    int neg_form;
    float neg_halfmax;
    float feedback;
    float subs_rate;

};

struct diffusion_type {
    SEGMENT_TYPE
    double D;
    double difconc1, difconc2;
};

struct flux_type {
    SEGMENT_TYPE
    double maxflux;
    double fluxconc1, fluxconc2;
    int power; /* used for exponent for CICR flux */
};

struct pump_type {
    SEGMENT_TYPE
    double max_rate;
    double half_conc;
    double half_conc_pow;
    double moles_in, moles_out;
    int power;
};

/* 1-D or 2-D reaction pool object. */
struct rxnpool_type {
    SEGMENT_TYPE
    double	Conc;
    double      quantity;
    double	previous_state;
    double	Cmin;
    double	Cinit;
    double      Qinit;
    double	len;   /* Use for 1D or 2D  cylinder, not sphere */
    double      radius; /* use for 1D, 2D cylinder or sphere */
    double	vol;
    double	SAside; /* Use for 1D or 2D cylinder, not sphere */
    double	SAout; /* use for 2D cyl or sphere, not 1D cylinder */
    double	SAin;
    int         type;
    double      units;
    double      Iunits;
};

struct IP3R_type {
	SEGMENT_TYPE
	double fraction;
	double previous_state;
	double alpha;
	double beta;
	double gamma;
	int alpha_state;
	int beta_state;
	int gamma_state;
	int conserve;
	double xinit;
	double xmin;
	double xmax;
};
