//genesis
//chemlib.g
/*
************************************************************************
**                      SHELL COMMANDS                    	     **
************************************************************************
*/

/*
*************************************************************************
**                              OBJECT TABLE                           **
*************************************************************************
*/

//==========================================================================
//	name		data            function	class	
//==========================================================================

object	diffusion	diffusion_type	diffusion	segment channel\
	-author	  "A. Blackwell GMU Dec 97; revised GMU Dec 99" \
	-actions  INIT PROCESS RESET CHECK \
	-messages POOL1	  0  3  len1 area1 conc1 \ /* pool 1 dimensions */
		  POOL2   1  3  len2 area2 conc2 \ /* pool 2 dimensions*/
	-fields	"D = diffusion constant in cm^2/s" \
		"difconc1 = D*area/len*conc21 send value to B field of pool 1, A field of pool 2" \
		"difconc2 = D*area/len*conc2, send value to A field of pool 1, B field of pool 2" \
	-description "Computes flux in mmoles due to diffusion through surface between two rxnpools" \
		"(1-D or 2-D). Input must be concentration (not quantity). return messages:"\
		" addmsg from diffusion object to pool1 DIFFUSION difconc2 difconc1 " \
		" addmsg from diffusion object to pool2 DIFFUSION difconc1 difconc2 "

object	rxnpool		rxnpool_type 	rxnpool		segment concentration \
	-author	  "A. Blackwell ERIM Aug 96; revised GMU Dec 99" \
	-actions  INIT PROCESS RESET CHECK SAVE2 RESTORE2 \
	-messages RXN0  0  1    A  \   /* type 0: 0th order rxn, concentration units*/
		  RXN1  1  1    B  \   /* type 1: 1st order rxn, concentration units */
		  RXN2  2  2    A  B  \  /* type 2: 2nd order rxn, concentration units */
		  PUMP  3  2    kpump Ceq \ /* type 3: interaction with pump, mmole input */
		  CURRENT 4 2   charge current \ /* type 4: electrical current, mmole input */
		  DIFFUSION 5 2 A B \ /* type 5: flow due to diffusion or CICR, mmole input */
		  MMPUMP 6 1	A \ /* type 6: 0th order rxn, mmole input */
		  RXN2MOLES 7 2 A B \ /* type 7: 2nd order rxn, mmole input */
	-fields	"Conc = concentration (integrated)" \
		"Cmin = minimum concentration" \
		"Cinit = initial concentration" \
		"quantity = quantity (mole)" \
		"Qinit = initial quantity (mole)" \
		"len = length of cylindrical pool" \
		"radius = radius of cylindrical or spherical pool" \
		"vol = volume of pool in Liters" \
		"SAside = side surface area of cylinder" \
		"SAout = outer surface area of spherical shell" \
		"SAin = inner surface area of spherical shell" \
		"type = whether pool integrates concentration or quantity"\
		"units = 1 for mole, 1e-3 for mmole, 1e-6 for umole, etc."\
		"Iunits = 1 for Amp,sec, 1e-12 for nA, msec"\
	-description "Concentration pool. 1-D or 2-D. Interacts with reactions, diffusion, pumps," \
		"current, CICR or IICR.  Accumulates fluxes into A and B to solve d[C]/dt = A - B*[C]"\ 
		"Input in Concentration for RXN0, RXN1 and RXN2 (0th, 1st and 2nd order reactions." \ 
		"Input from CURRENT requires correct unit fields:" \
		"	Iunits = 1 for Amp,sec, 1e-12 for nA, msec"\
		"	units = 1 for mole, 1e-3 for mmole, 1e-6 for umole, etc."\
		"Input from taupump (PUMP) is Kpump * SA (units of vol/time) and concentration" \
		"Input from 1-D or 2-D diffusion or CICR flux : DIFFUSION " \
		"Input from Michaelis-Menton enzyme or M-M pump : MMPUMP "\
		"Input for RXN2MOLES (second order reaction, in quantity units"\
		"If type = 0, converts quantity inputs to concentration and then integrates."\
		"If type = 1, converts concentration inputs to quantity and then integrates."\
		"Any system of units are allowed, but must be consistent across all objects."\
		"If using mmole, then all other objects that interact with rxnpool must use mmole units."\
		"If use m^3 units for volume, then conc units are mM when quantity is mole units."


object	cicr		IP3R_type	cicrpool	segment gate\
	-author   "A. Blackwell ERIM Aug 96; revised GMU Aug 99" \
	-actions  INIT PROCESS RESET CHECK \
	-messages ASTATE 0 2    kbalpha conc \ /* back rate const and conc in alpha dir */
		  BSTATE 1 2    kbbeta  conc \ /* back rate const and conc in beta dir */
		  GSTATE 2 2    kbgamma conc \ /* back rate const and conc in gamma dir */
		  IP3    3 1    ip3conc      \ /* concentration of ip3 */
		  CALCIUM 4 1   caconc       \ /* concentration of calcium */
		  CONSERVE 5 1  xconc	     \ /* conc of other states */
	-fields "fraction = fraction of channel in state "\
	        "alpha = forward rate constant in alpha direction "\
	        "beta = forward rate constant in beta direction "\
	        "gamma = forward rate constant in gamma direction "\
	        "alpha_state = if ip3 is bound (1) or not (0) for this state "\
	        "beta_state = if calcium is bound (1) or not (0) to open part for this state "\
	        "gamma_state = if calcium is bound(1) or not (0) to closed part for this state "\
	        "xinit = initial value"\
	        "xmin = minimum value"\
	        "xmax = maximum value"\
	        "conserve = flag indicating a conserve pool"\
	 -description " computes state variables associated with ip3 receptor channel."\
	 	      " Needs to send messages to cicrflux, which computes flux between two "\
	 	      " pools connected by the channel (i.e. cytosol and ER). See script for usage."

object	cicrflux	flux_type	cicrflux	segment channel\
	-author	  "A. Blackwell ERIM Aug 96; revised GMU Dec 99" \
	-actions  INIT PROCESS RESET CHECK \
	-messages CONC1	 0  1   conc1 \ /* type 0: pool 1 concentration */
		  CONC2  1  1   conc2 \ /* type 1: pool 2 concentration*/
		  IP3R   2  1   ip3r  \ /* type 2: pool 2 fraction ip3r conducting*/
	-fields	"maxflux = single channel flux (cm/msec) * number of channels * Surface area (cm^2)" \
		"power = exponent; fraction open = ip3r^type" \
		"fluxconc1 = maxflux*openfraction*conc1, send value to B of rxnpool 1, A of rxnpool 2" \
		"fluxconc2 = maxflux*openfraction*conc2, send value to A of rxnpool 1, B of rxnpool 2" \
	-description "Computes flow through ip3 receptor channel, between ER and cytosol rxnpools." \
		"Requires inputs from two rxnpools (units must be concentration) and the ip3r object "\
		"return messages as follows: "\
		"addmsg from cicrflux object to pool1 DIFFUSION fluxconc2 fluxconc1 " \
		"addmsg from cicrflux object to pool 2 DIFFUSION fluxconc1 fluxconc2 "

object	enzyme		enzyme_type	enzyme		segment \
	-author	  "A. Blackwell ERIM Aug 96; revised GMU Dec 99" \
	-actions  INIT PROCESS RESET SET\
	-messages ENZ   0   1   conc \  /* concentration or weight or mmol of enzyme */
		  FEEDBACK 1 1  conc \ /* conc. of feedback substance */
		  SUBSTRATE 2 1  conc \ /* conc. of substrate */
	-fields	  "Vmax = maximal rate of reaction, units of mmole per msec-(mmol or mg or conc enz)" \
		  "Km = combination rate constant: (kf2+kb1)/kf1:" \
		  "Product = mmoles of product produced" \
		  "thresh = below this conc, pos feedback, above -> neg fb." \
		  "pos_pow = raise pos fb to this power " \
		  "pos_form = 0 for hyperbolic, 1 for sigmoid " \
		  "pos_halfmax = param for sigmoid feedback " \
		  "neg_pow = raise neg fb to this power " \
		  "neg_form = 0 for hyperbolic, 1 for sigmoid " \
		  "neg_halfmax = param for sigmoid feedback " \
	     	  "feedback = computed value of feedback " \
	-description "Simplified Michaelis-Menton enzyme kinetics.  Can assume constant substrate, "\
		"or can compute effect of substrate on reaction rate.  Vmax = kf2; Km = (Kf2+kb1)/kf1"\
		"			kf1	  kf2"\
		"[Enzyme] + [Substrate] <-> [E-S] -> [Enzyme] + [Product]"\
		"			 kb1"\
		"Under assumptions of M-M dynamics, Solves dprod/dt =  kf2*Enzyme*Subs/(Subs + Km)"\
		"Vmax units are quantity of product per (quantity or concen of enzyme) per time.  Must be "\
		"consistent with Enzyme units (input from enzyme rxnpool.  Substrate and Km "\
		"must have units of Conc.  Product quantity = Vmax * E * S/ (S+Km)."\  
		" addmsg from enz object to rxnpool MMPUMP product."\
		"For more accurate MM kinetics, use reaction objects OR enz object in kinetikit."\
	        "Feedback allows decrease in enzyme activity as function of "\
		"concentration of feedback substance " \  
		"hyperbolic form: fb propto 1 / conc(feedback substance) for neg feedback " \
	        "                 fb propto conc (feeback substance) for positive feedback. " \
    		"Sigmoid form: fb propto conc/conc + halfmax (for neg feedback) " \
                "		fb propto halfmax / conc + halfmax (for pos feedback) "
	
object	mmpump		pump_type	mmpump		segment  channel\
	-author	  "A. Blackwell ERIM Aug 96; revised GMU Dec 99" \
	-actions  INIT PROCESS RESET SET\
	-messages CONC   0   1   conc \  /* concentration */
	-fields	  "max_rate = Maximum Pump Rate"\
		  "moles_in = State variable for return message to pools" \
		  "moles_out = State variable for return message to pools" \
		  "half_conc = concentration at which pump rate is half maximal"\
		  "power = exponent applied to Concentration"\
	-description " A Michaelis-Menton type of formulation of pump.  Either SERCA ATPase pump or "\
		     " Na-Ca ATPase cytosolic pump. Requires input message in concentration units."\
		     " addmsg from mmpump to cytosol-rxnpool MMPUMP moles_out."\
		     " addmsg from mmpump to ER or extracellular rxnpool MMPUMP moles_in."\
		     " Computes right hand side of dC/dt = (+/-)max_rate * Ca^n / (Ca^n + half_conc^n). "
		  
object	reaction	react_type	reaction	segment \
	-author	  "A. Blackwell ERIM Aug 96 ; revised GMU Dec 99" \
	-actions  INIT PROCESS RESET SET\
	-messages SUBSTRATE 0   1   conc \  /* concentration or quantity of substrates */
		  PRODUCT   1   1   conc \  /* concentration or quantity of products */
	-fields	  "kf = forward rate constant"\
		  "kb = backward rate constant"\
		  "kbprod = kb * products" \
		  "kfsubs = kf * substrates" \
	-description "An element to compute kf*substrates and kb*products for nth order reactions."\
		"dC/dt = kf * substrate * substrate * ... - kb*product* ..."\  
		"For each substrate and product in reaction, a message must be received (n times "\
		"if n moles required). Input and rate constants both must be in units of concentration,"\
		"(use RXN2 to send message to rxnpool) or both must be in units of moles (use RXN2MOLE)."\
	   	    "addmsg from reaction to substrate rxnpool RXN2 (or RXN2MOLES) kbprod kfsubs"\ 
	   	    "addmsg from reaction to product rxnpool RXN2 (or RXN2MOLES) kfsubs kbprod	"

object conservepool	conserve_type	conservepool	segment concentration\
	-author "A. Blackwell ERIM Aug 96; revised GMU Dec 99" \
	-actions INIT PROCESS RESET \
	-messages CONC 	0   	1   	conc \ /* concentration of other forms */
		  MOLES	1	1	moles \ /* quantity of other forms */
	-fields   "Conc = concentration of molecule"\
		  "Cinit = initial concentration"\
		  "Ctot = total concentration of this and other forms of molecule" \
		  "quantity = moles of molecule"\
		  "Qinit = initial quantity of molecule"\
		  "Qtot = total quantity of this and other forms of molecule"\
		  "Cmin = minimum concentration"\
		  "type = whether specifying quantity or concentration of molecule"\
		  "volume: used to convert from concentration to quantity and vice versa"\
	-description "Implementation of pool for conserved molecules. If type = 0, computes"\
		"concentration of molecule which is the total conc less the conc in all"\
		"other states. If type = 1, computes quantity of molecule which is the "\
		"total quantity less the quantity in all other states.  Then computes"\
 		"concentration by dividing by volume."

