/* conservepool.c */
#include "chem_ext.h"

/* Implementation of pool for conserved molecules */

/* computes concentration of molecule which is the total concentration
** less the concentration in all other states. */

/* Avrama Blackwell, ERIM 
   Modified Dec 1999 to compute quantities */

int conservepool(conspool,action)
register struct conserve_type *conspool;
Action		*action;
{
double	dt;
double conctot, moletot;
MsgIn 	*msg;
int	n;

    if(debug > 1){
	ActionHeader("conserverpool",conspool,action);
    }

    SELECT_ACTION(action){
    case INIT:
	conspool->previous_state = conspool->Conc;
	break;

    case PROCESS:
	dt = Clockrate(conspool);
	conctot = moletot = 0;

	/* Read the msgs to get the total conservation in other states */
	MSGLOOP(conspool,msg){
	    case 0:	/* conservation of mass */
	    	conctot += MSGVALUE(msg,0);
	    	break;
	    case 1:  /* moles of other forms */
	        moletot += MSGVALUE(msg,0);
	        break;
	}

	if (conspool->type == 0)   /* accumulates concentration */
	  {
	    conspool->Conc = conspool->Ctot - conctot;
	    conspool->quantity = conspool->Conc*conspool->volume;
	  }
	else /* accumulates moles */
	  {
	    conspool->quantity = conspool->Qtot - moletot;
	    conspool->Conc = conspool->quantity/conspool->volume;
	  }
	    if (conspool->Conc < conspool->Cmin)
	      {
	       conspool->Conc = conspool->Cmin;
	       conspool->quantity = conspool->Conc*conspool->volume;
	      }
	break;

	
    case RESET:
      if (conspool->type == 0)
	{
	  conspool->Conc = conspool->Cinit;
	  conspool->Qtot = conspool->Ctot*conspool->volume;
	  conspool->quantity = conspool->Conc*conspool->volume;
	}
      if (conspool->type == 1)
	{
	  conspool->quantity = conspool->Qinit;
	  conspool->Ctot = conspool->Qtot/conspool->volume;
	  conspool->Conc = conspool->quantity/conspool->volume;
	}
	break;
    }
    
    return(0);
    
}

